#!/bin/bash

set -e -o pipefail
INSTALL_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

echo "PWD_DIR: $PWD_DIR"
echo "BASH_SOURCE: ${BASH_SOURCE[0]}"
echo "Install_DIR: $INSTALL_DIR"

usage() {
	echo "Usage: $(basename "$0") root_dir [local_repo]"
	echo "  root_dir: directory containing serums-backend, and serums-blockchain"
	echo "  local_repo: ip and address of the local docker registry. Defaults to localhost:55000"
	echo "  Note: Assumes that cryptographic materials have been generated (see serums_manager.sh init)"
	echo "  and that the connection-profile.json (in serums-blockchain/config-maps/) has been updated accordingly"
	exit 1
}

if [ "$#" -lt 1 ]; then
    echo "Illegal number of parameters"
    usage
fi

ROOT_DIR="$1"
echo "ROOT_DIR: $ROOT_DIR"
# Script will set the Docker repository either to the provided value, or default to Docker Registry (as mentioned in the README.md)
LOCAL_REPO="${2:-localhost:55000}"
echo "Using local repo $LOCAL_REPO"

# Setting up the backend
cd $ROOT_DIR/serums-backend
docker build . -t serums/backend
docker tag serums/backend $LOCAL_REPO/serums-backend:latest
docker push $LOCAL_REPO/serums-backend

# Setting up the frontend (currently not used for solution)
# cd $ROOT_DIR/serums-frontend
# docker build . -t serums/frontend
# docker tag serums/frontend $LOCAL_REPO/serums-frontend:latest
# docker push $LOCAL_REPO/serums-frontend
# cd $INSTALL_DIR

# Setting up the blockchain explorer (currently only used for configmaps for API <-> Blockchain communication)
cd $ROOT_DIR/serums-blockchain/config-maps/
# This script will create the configmaps for API <-> Blockchain communication (connection-profile.json)
./create-config-map.sh
cd $INSTALL_DIR

# Apply the K8s deployment for the parts you want to deploy
kubectl apply -f backend-template.yaml
# kubectl apply -f explorer-template.yaml
# kubectl apply -f frontend-template.yaml
# ./create_ingress.sh

# HLF Stuff - you can make this deploy the blockchain as well.
# cd $ROOT_DIR/serums-blockchain
# for entry in "$ROOT_DIR/serums-blockchain"/*
# do
#   echo "entry: $entry"
# done
# FABRIC_DIR=$PWD
# echo "PWD: $PWD"
# ./serums_manager.sh install
# ./serums_manager.sh channels
# ./serums_manager.sh chaincodes
