const Yup = require('yup')

Yup.addMethod(Yup.array, 'unique', function (message, mapper = (a) => a) {
    return this.test(
        'unique',
        message,
        (list) => !list || list.length === new Set(list.map(mapper)).size
    )
})

const personType = Yup.string().oneOf(['GROUP', 'INDIVIDUAL', 'DEPARTMENT'])

const attribute = {
    name: Yup.string().required()
}

const attributes = Yup.array()
    .of(Yup.object().shape(attribute))
    .min(1)
    .unique('Duplicate attribute', (a) => a.name)

const rule = {
    action: Yup.string().oneOf(['ALLOW', 'DENY']).required(),
    grantor: Yup.object().shape({
        type: personType.required(),
        // If this is provided, must match the userId or user is in ADMIN group
        id: Yup.string().notRequired()
    }),
    grantee: Yup.object().shape({
        type: personType.required(),
        id: Yup.string().required(),
        orgId: Yup.string().required()
    }),
    access: attributes.required(),
    requestor: Yup.string().notRequired(),
    expires: Yup.date().min(new Date(), 'expires is in the past').required()
}

const ruleIdRegex = new RegExp('^RULE_.+')
const proofIdRegex = new RegExp('^PROOF_.+')
const ruleId = Yup.string().matches(ruleIdRegex) // TODO add regex to match rule format
const proofId = Yup.string().matches(proofIdRegex)
const jwtToken = Yup.string().required()

const accessRequest = {
    patientId: Yup.string().required(),
    patientOrgId: Yup.string().required(),
    access: attributes.required()
}


const accessLogFilter = {
    filterType: Yup.string().oneOf(['SIMPLE']).required(),
    // TODO key and value should be required if filterType is Simple
    key: Yup.string().oneOf(
        [
            'requestor.individualId',
            'requestor.groupId',
            'requestor.orgId',
            'target.patientId',
            'target.patientOrgId'
        ]
    ).required(),
    value: Yup.string().required()
}

const accessLogFilters = {
    filters: Yup.array().of(Yup.object().shape(accessLogFilter)).unique('duplicate attribute', (f) => f.key).ensure()
}

const getRulesFilter = {
    filterType: Yup.string().oneOf(['SIMPLE', 'NOT_EXPIRED']).required(),
    key: Yup.string().oneOf(
        [
            'id',
            'action',
            'grantor.type',
            'grantor.id',
            'grantor.orgId',
            'grantee.type',
            'grantee.id',
            'grantee.orgId'
        ]
    ),
    value: Yup.string()
}

const getRulesFilters = {
    filters: Yup.array().of(Yup.object().shape(getRulesFilter)).unique('duplicate attribute', (f) => f.key).ensure()
}

const create_proof = {
    ruleId: Yup.string().required(),
    serumsId: Yup.string().required(),
    hospitalIds: Yup.array().of(Yup.string().oneOf(['USTAN', 'ZMC', 'FCRB'])).required(),
    date: Yup.string()
}

const update_proof = {
    id: Yup.string().required(),
    type: Yup.string().oneOf(['request_received', 'data_vault_created', 'data_selected', 'data_filled', 'encryption', 'cleanup']).required(),
    hospitalId: Yup.string().oneOf(['USTAN', 'ZMC', 'FCRB']),
    status: Yup.string().required(),
    content: Yup.object(),
    date: Yup.string().required()
}

module.exports = {
    jwtToken,
    personType,
    attribute,
    attributes,
    rule,
    ruleId,
    proofId,
    accessRequest,
    accessLogFilters,
    getRulesFilters,
    ruleIdRegex,
    proofIdRegex,
    create_proof,
    update_proof
}
