#!/usr/bin/env python3

import yaml
import json

from os import listdir, mkdir, system
from os.path import isfile, join, basename, isdir

import argparse

def ensure_dir(dir):
    try:
        # Create target Directory
        mkdir(dir)
        print("Directory " , dir ,  " Created ")
    except FileExistsError:
        print("Directory " , dir ,  " already exists")

def get_files(dir):
    return [f for f in listdir(dir) if isfile(join(dir, f))]

def get_dirs(dir):
    return [join(dir, d) for d in listdir(dir) if isdir(join(dir, d))]

def create_hfc_keystore(configtx_path, hfc_keystore_path, parent_of_crypto_config):
    system('rm -rf {}'.format(hfc_keystore_path))
    ensure_dir(hfc_keystore_path)


    with open(configtx_path, 'r') as stream:
        try:
            configtx = yaml.safe_load(stream)
        except yaml.YAMLError as exc:
            print(exc)

    for org_data in configtx['Organizations']:
        all_user_dir = join(join(parent_of_crypto_config, org_data['MSPDir']), '../users')
        mspid = org_data['ID']
        for user_dir in get_dirs(all_user_dir):
            save_hfc_for_user(user_dir, mspid, hfc_keystore_path)

def save_hfc_for_user(user_dir, mspid, hfc_keystore_path):
    username = basename(user_dir)
    user_msp = join(user_dir, 'msp')
    admincerts_path = join(user_msp, 'admincerts')
    certificate_path = join(admincerts_path, get_files(admincerts_path)[0])
    keystore_path = join(user_msp, 'keystore')
    signing_identity = get_files(join(user_dir, 'msp/keystore'))[0].split('_')[0]

    with open(certificate_path, 'r') as stream:
        certificate = stream.read()
        certificate_no_lines = certificate.replace('\n', '\\n')


    user_hfc = {
        "name": username,
        "mspid": mspid,
        "roles": None,
        "affiliation": "",
        "enrollmentSecret": "",
        "enrollment": {
            "signingIdentity": signing_identity,
            "identity": {
                "certificate": certificate
            }
        }
    }
    print(hfc_keystore_path)
    print(username)
    with open(join(hfc_keystore_path, username), 'w') as stream:
        json.dump(user_hfc, stream, indent=2)

    with open(join(hfc_keystore_path, signing_identity + '-pub'), 'w') as stream:
        stream.write(certificate)

    with open(join(user_dir, 'msp/keystore/{}_sk'.format(signing_identity)), 'r') as stream:
        private_key = stream.read()

    with open(join(hfc_keystore_path, signing_identity + '-priv'), 'w') as stream:
        stream.write(private_key)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Creates the HFC keystore')
    parser.add_argument('configtx_path')
    parser.add_argument('hfc_keystore_path')
    parser.add_argument('parent_of_crypto_config')
    ARGS = parser.parse_args()

    create_hfc_keystore(ARGS.configtx_path, ARGS.hfc_keystore_path, ARGS.parent_of_crypto_config)

